#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

"""
Tests for casa_envoy Configure
"""
import os
from pathlib import Path
from unittest.mock import patch

from casa_envoy.configure import CasaConfigure, StandardRunner, ParallelRunner

cal_parameters = {
    "useCasa": False,
    "homeForReprocessing": "/home/casa/packages/pipeline/current",
    "rootDirectory": "/tmp/workspaces_tmp/",
    "processingDirectory": "tmpiox5trbp",
    "metadata": "test/input_files/test.json",
    "ppr": "test/input_files/PPR.xml",
    "product_type": "standard-cal",
    "parent_path": Path("I/am/a/path"),
}
parallel_parameters = {
    "useCasa": False,
    "homeForReprocessing": "/home/casa/packages/pipeline/current",
    "rootDirectory": "/tmp/workspaces_tmp/",
    "processingDirectory": "tmpiox5trbp",
    "metadata": "test/input_files/test.json",
    "ppr": "test/input_files/PPR.xml",
    "product_type": "standard-cal",
    "parent_path": Path("I/am/a/path"),
    "requested_parallel": "4",
}


class TestCasaConfigure:
    """
    Test methods for CASA environment set up
    """

    def test_setup_environment(self):
        CasaConfigure(parameters=cal_parameters).setup_environment()
        assert os.environ.get("SCIPIPE_ROOTDIR") == cal_parameters["rootDirectory"]
        assert os.environ.get("CASA_HOME") == cal_parameters["homeForReprocessing"]
        assert os.environ.get("PPR_FILENAME") == "test/input_files/PPR.xml"


class TestStandardRunner:
    """
    Test methods for Standard CASA calls
    """

    @patch("os.chdir")
    @patch("subprocess.Popen")
    def test_std_call_casa(self, mock_subprocess, mock_chdir):
        StandardRunner(parameters=cal_parameters).call_casa()
        assert mock_subprocess.called_with(
            "PYTHONPATH='' xvfb-run -e ${PWD}/xvfb-run.err.txt -d -s \"-screen 0 800x600x16\" "
            "${CASA_HOME}/bin/casa --pipeline --nogui --nologger -c "
            "${CASA_HOME}/pipeline/pipeline/runvlapipeline.py ${PPR_FILENAME} || true"
        )


class TestParallelRunner:
    """
    Test methods for Parallel CASA calls
    """

    @patch("os.chdir")
    @patch("subprocess.Popen")
    def test_parallel_call_casa(self, mock_subprocess, mock_chdir):
        ParallelRunner(parameters=parallel_parameters).call_casa()
        assert mock_subprocess.called_with(
            "PYTHONPATH='' xvfb-run -e ${PWD}/xvfb-run.err.txt -d -s \"-screen 0 800x600x16\" "
            "${CASA_HOME}/bin/mpicasa -n 4 ${CASA_HOME}/bin/casa --pipeline --nogui --nologger -c "
            "${CASA_HOME}/pipeline/pipeline/runvlapipeline.py ${PPR_FILENAME} || true"
        )
